"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setAuthorName = exports.sortPackageJson = exports.buildPackage = exports.composePackageJson = exports.createPackageJson = exports.copyPackageTemplate = exports.getSafePackageName = exports.getPackageTemplateType = exports.chooseTemplatePrompt = exports.getAuthor = exports.getAuthorName = exports.safePackageName = void 0;
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var shelljs_1 = require("shelljs");
var enquirer_1 = require("enquirer");
var utils_1 = require("../../shared/utils");
var generate_1 = require("./setup/generate");
var messages_1 = require("../../shared/messages");
var package_const_1 = require("../../shared/constants/package.const");
exports.safePackageName = function (name) {
    return name
        .toLowerCase()
        .replace(/(^@.*\/)|((^[^a-zA-Z]+)|[^\w.-])|([^a-zA-Z0-9]+$)/g, '');
};
function getAuthorName() {
    var author = '';
    author = shelljs_1.exec('npm config get init-author-name', {
        silent: true
    }).stdout.trim();
    if (author) {
        return author;
    }
    author = shelljs_1.exec('git config user.name', { silent: true }).stdout.trim();
    if (author) {
        setAuthorName(author);
        return author;
    }
    author = shelljs_1.exec('npm config get init-author-email', {
        silent: true
    }).stdout.trim();
    if (author) {
        return author;
    }
    author = shelljs_1.exec('git config user.email', { silent: true }).stdout.trim();
    if (author) {
        return author;
    }
    return author;
}
exports.getAuthorName = getAuthorName;
exports.getAuthor = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var author, licenseInput;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                author = getAuthorName();
                if (!!author) return [3 /*break*/, 2];
                licenseInput = new enquirer_1.Input({
                    name: 'author',
                    message: 'Who is the package author?'
                });
                return [4 /*yield*/, licenseInput.run()];
            case 1:
                author = _a.sent();
                _a.label = 2;
            case 2: return [2 /*return*/, author];
        }
    });
}); };
exports.chooseTemplatePrompt = new enquirer_1.Select({
    message: 'Choose a template',
    choices: Object.keys(generate_1.generateSetup).map(function (option) { return ({
        name: option,
        message: utils_1.info(option)
    }); })
});
exports.getPackageTemplateType = function (template, onInvalidTemplate) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
        if (template !== undefined && generate_1.generateSetup[template] === undefined) {
            onInvalidTemplate();
        }
        if (template === undefined || generate_1.generateSetup[template] === undefined) {
            return [2 /*return*/, exports.chooseTemplatePrompt.run()];
        }
        return [2 /*return*/, template];
    });
}); };
exports.getSafePackageName = function (_a, onFailedPath) {
    var workspaceRoot = _a.workspaceRoot, packageSetupPath = _a.packageSetupPath, packageName = _a.packageName;
    return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var isExist, packageNamePrompt, newPackageName;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, fs_extra_1.default.pathExists(workspaceRoot + "/" + packageSetupPath + "/" + packageName)];
                case 1:
                    isExist = _b.sent();
                    if (!isExist) {
                        return [2 /*return*/, packageName];
                    }
                    onFailedPath(packageName);
                    packageNamePrompt = new enquirer_1.Input({
                        message: messages_1.generateMessage.exists(packageName),
                        initial: messages_1.generateMessage.copy(packageName),
                        result: function (v) { return v.trim(); }
                    });
                    return [4 /*yield*/, packageNamePrompt.run()];
                case 2:
                    newPackageName = _b.sent();
                    return [2 /*return*/, exports.getSafePackageName({
                            packageName: newPackageName,
                            workspaceRoot: workspaceRoot,
                            packageSetupPath: packageSetupPath
                        }, onFailedPath)];
            }
        });
    });
};
exports.copyPackageTemplate = function (_a) {
    var dir = _a.dir, template = _a.template;
    return fs_extra_1.default.copy(path_1.default.resolve(__dirname, "../../../../templates/generate/" + template), dir, {
        overwrite: true
    });
};
exports.createPackageJson = function (_a) {
    var dir = _a.dir, preset = _a.preset;
    return fs_extra_1.default.outputJSON(path_1.default.resolve(dir, package_const_1.PACKAGE_JSON), preset, {
        spaces: 2
    });
};
exports.composePackageJson = function (_a) {
    var author = _a.author, name = _a.name, hostName = _a.hostName, license = _a.license, template = _a.template;
    var slashNameIndex = hostName.indexOf('/');
    var namespace = slashNameIndex === -1 ? "@" + hostName : hostName.slice(0, slashNameIndex);
    var safeName = exports.safePackageName(name);
    var packageName = namespace ? namespace + "/" + safeName : safeName;
    var packageJson = tslib_1.__assign(tslib_1.__assign({}, template.packageJson), { name: packageName, author: author });
    if (license) {
        packageJson.license = license;
    }
    return packageJson;
};
exports.buildPackage = function () { return shelljs_1.exec('yarn build', { silent: true }); };
exports.sortPackageJson = function () {
    return shelljs_1.exec('npx sort-package-json', { silent: true });
};
function setAuthorName(author) {
    shelljs_1.exec("npm config set init-author-name \"" + author + "\"", { silent: true });
}
exports.setAuthorName = setAuthorName;
